@@ -26,6 +26,7 @@ import os
 import sys
 import time
 import timeit
+import string
 from contextlib import contextmanager
 from .ports import BaseOutput
 from .types import encode_variable_int
@@ -46,11 +47,26 @@ class ByteReader(object):
         self.pos = 0
         self._eof = EOFError('unexpected end of file')
 
+    def _print_bytes(self, n):
+        """Print the next n bytes as hex and characters.
+
+        This is used for debugging.
+        """
+        data = self._buffer[self.pos:self.pos + n]
+        for pos, byte in enumerate(data, start=self.pos):
+            char = chr(byte)
+            if not char in string.printable or char in string.whitespace:
+                char = ''
+            print('  {:06x}: {:02x} {}'.format(pos, byte, char))
+
+        if len(data) < n:
+            raise EOFError('end of file reached in read_list()')
+
     def read_byte(self):
         """Read one byte."""
         try:
             byte = self._buffer[self.pos]
-            # print('{:02x}'.format(byte))
+            # self._print_bytes(1)
             self.pos += 1
             return byte
         except IndexError:
@@ -67,10 +83,9 @@ class ByteReader(object):
 
     def read_list(self, n):
         """Read n bytes and return as a list."""
+        # self._print_bytes(n)
         i = self.pos
         ret = self._buffer[i:i + n]
-        # for byte in ret:
-        #     print('  {:02x} {!r}'.format(byte, chr(byte)))
         if len(ret) < n:
             raise self._eof
 
@@ -257,13 +272,15 @@ class MidiFile:
         length = self._file.read_long()
         start = self._file.pos
         last_status = None
-        last_systex = None
+        last_sysex = None
 
         while 1:
             # End of track reached.
             if self._file.pos - start == length:
                 break
 
+            # print('--- New')
+
             delta = self._read_variable_int()
 
             # Todo: not all messages have running status
@@ -274,7 +291,9 @@ class MidiFile:
                 status_byte = last_status
             else:
                 status_byte = self._file.read_byte()
-                last_status = status_byte
+                if status_byte != 0xff:
+                    # Meta messages don't set running status.
+                    last_status = status_byte
 
             if status_byte == 0xff:
                 message = self._read_meta_message()
@@ -283,7 +302,7 @@ class MidiFile:
                 last_sysex = message
             elif status_byte == 0xf7:
                 # Todo: check if this works as intended.
-                if last_systex is None:
+                if last_sysex is None:
                     raise IOError(
                         'sysex continuation without preceding sysex')
                 last_sysex.data += self._read_sysex()
@@ -294,6 +313,8 @@ class MidiFile:
             message.time = delta
             track.append(message)
 
+            # print('---', message)
+
             if message.type == 'end_of_track':
                 break
 
